@@ -17,16 +17,29 @@
 
 package org.apache.commons.cli;
 
+import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.when;
 
 import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.io.UncheckedIOException;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.stream.Stream;
 
 import org.apache.commons.cli.HelpFormatter.Builder;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
+
 
 /**
  * Test case for the HelpFormatter class.
@@ -34,6 +47,46 @@ import org.junit.jupiter.api.Test;
 public class HelpFormatterTest {
     private static final String EOL = System.lineSeparator();
 
+    static Stream<Arguments> deprecatedOptionsProvider() {
+        final List<Arguments> lst = new ArrayList<>();
+        Option option = Option.builder("a").longOpt("aaa").desc("dddd dddd dddd")
+                .deprecated(DeprecatedAttributes.builder().setForRemoval(true).setSince("now")
+                        .setDescription("Why why why").get())
+                .build();
+
+        HelpFormatter hf = HelpFormatter.builder().get();
+        lst.add(Arguments.of(hf, option, "[Deprecated] dddd dddd dddd"));
+
+
+        hf = HelpFormatter.builder().setShowDeprecated(false).get();
+        lst.add(Arguments.of(hf, option, "dddd dddd dddd"));
+
+        hf = HelpFormatter.builder().setShowDeprecated(true).get();
+        lst.add(Arguments.of(hf, option, "[Deprecated] dddd dddd dddd"));
+
+        hf = HelpFormatter.builder().setShowDeprecated(o -> String.format("%s [%s]", HelpFormatter.getDescription(o), o.getDeprecated())).get();
+        lst.add(Arguments.of(hf, option, "dddd dddd dddd [Deprecated for removal since now: Why why why]"));
+
+        option = Option.builder("a").longOpt("aaa")
+                .deprecated(DeprecatedAttributes.builder().setForRemoval(true).setSince("now")
+                        .setDescription("Why why why").get())
+                .build();
+
+        hf = HelpFormatter.builder().get();
+        lst.add(Arguments.of(hf, option, "[Deprecated]"));
+
+        hf = HelpFormatter.builder().setShowDeprecated(false).get();
+        lst.add(Arguments.of(hf, option, ""));
+
+        hf = HelpFormatter.builder().setShowDeprecated(true).get();
+        lst.add(Arguments.of(hf, option, "[Deprecated]"));
+
+        hf = HelpFormatter.builder().setShowDeprecated(o -> String.format("%s [%s]", HelpFormatter.getDescription(o), o.getDeprecated())).get();
+        lst.add(Arguments.of(hf, option, "[Deprecated for removal since now: Why why why]"));
+
+        return lst.stream();
+    }
+
     @Test
     public void testAccessors() {
         final HelpFormatter formatter = new HelpFormatter();
@@ -140,9 +193,9 @@ public class HelpFormatterTest {
         //@formatter:off
         assertEquals(
                 "usage: foobar" + EOL +
-                "" + EOL +
+                EOL +
                 "Header" + EOL +
-                "" + EOL +
+                EOL +
                 "Footer" + EOL,
                 out.toString());
         //@formatter:on
@@ -171,9 +224,9 @@ public class HelpFormatterTest {
         //@formatter:off
         assertEquals(
                 "usage: foobar" + EOL +
-                "" + EOL +
+                EOL +
                 "Header" + EOL +
-                "" + EOL +
+                EOL +
                 "Footer" + EOL,
                 out.toString());
         //@formatter:on
@@ -182,9 +235,9 @@ public class HelpFormatterTest {
         //@formatter:off
         assertEquals(
                 "usage: foobar" + EOL +
-                "" + EOL +
+                EOL +
                 "Header" + EOL +
-                "" + EOL +
+                EOL +
                 "Footer" + EOL,
                 out.toString());
         //@formatter:on
@@ -193,9 +246,9 @@ public class HelpFormatterTest {
         //@formatter:off
         assertEquals(
                 "usage: foobar" + EOL +
-                "" + EOL +
+                EOL +
                 "Header" + EOL +
-                "" + EOL +
+                EOL +
                 "Footer" + EOL,
                 out.toString());
         //@formatter:on
@@ -204,9 +257,9 @@ public class HelpFormatterTest {
         //@formatter:off
         assertEquals(
                 "usage: foobar" + EOL +
-                "" + EOL +
+                EOL +
                 "Header" + EOL +
-                "" + EOL +
+                EOL +
                 "Footer" + EOL,
                 out.toString());
         //@formatter:on
@@ -360,6 +413,26 @@ public class HelpFormatterTest {
         //@formatter:on
     }
 
+    @ParameterizedTest
+    @MethodSource("deprecatedOptionsProvider")
+    public void testPrintDeprecatedOptions(final HelpFormatter hf, final Option option, final String expectedTxt) {
+        final StringBuffer sb = new StringBuffer();
+
+        final int leftPad = 1;
+        final int descPad = 3;
+        final String lpad = hf.createPadding(leftPad);
+        final String dpad = hf.createPadding(descPad);
+        Options options;
+        final StringBuilder expected = new StringBuilder().append(lpad).append("-a,--aaa");
+
+        options = new Options().addOption(option);
+        if (expectedTxt.length() > 0) {
+            expected.append(dpad).append(expectedTxt);
+        }
+        hf.renderOptions(sb, 160, options, leftPad, descPad);
+        assertEquals(expected.toString(), sb.toString());
+    }
+
     @Test
     public void testPrintHelpNewlineFooter() {
         final HelpFormatter formatter = new HelpFormatter();
@@ -424,6 +497,22 @@ public class HelpFormatterTest {
         assertThrows(IllegalArgumentException.class, () -> formatter.printHelp("", new Options(), false), "null command line syntax should be rejected");
     }
 
+    @Test
+    public void testPrintHelpWithSince() {
+        final String [] expected = {"usage: Command syntax", "Header", "Options            Since   Description",
+                "  -n,--no-since    -          Description for n", "  -W,--with-since  1.19.0     Descripton for W", "footer"};
+        final Options options = new Options()
+                .addOption(Option.builder("W").longOpt("with-since").since("1.19.0").desc("Descripton for W").build())
+                .addOption(Option.builder("n").longOpt("no-since").desc("Description for n").build());
+
+        final HelpFormatter formatter = HelpFormatter.builder().setShowSince(true).get();
+        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        try (PrintWriter pw = new PrintWriter(new OutputStreamWriter(baos))) {
+            formatter.printHelp(pw, 80, "Command syntax", "Header", options, 2, 5, "footer", false);
+        }
+        assertArrayEquals(expected, baos.toString().split(System.lineSeparator()));
+    }
+
     @Test
     public void testPrintOptionGroupUsage() {
         final OptionGroup group = new OptionGroup();
@@ -572,6 +661,24 @@ public class HelpFormatterTest {
         assertEquals("usage: app [-a] [-b] [-c]" + EOL, bytesOut.toString());
     }
 
+    @Test
+    public void testRenderSince() throws IOException {
+        final String[] expected = {"Options            Since   Description", "  -n,--no-since    -          Description for n",
+            "  -W,--with-since  1.19.0     Descripton for W"};
+        final Options options = new Options()
+                .addOption(Option.builder("W").longOpt("with-since").since("1.19.0").desc("Descripton for W").build())
+                .addOption(Option.builder("n").longOpt("no-since").desc("Description for n").build());
+        final HelpFormatter formatter = HelpFormatter.builder().setShowSince(true).get();
+
+        final StringBuffer sb = new StringBuffer();
+        formatter.renderOptions(sb, 50, options, 2, 5);
+        assertArrayEquals(expected, sb.toString().split(System.lineSeparator()));
+        // check internal exception handling for coverage
+        final HelpFormatter spy = spy(formatter);
+        when(spy.appendOptions(sb, 50, options, 2, 5)).thenThrow(IOException.class);
+        assertThrows(UncheckedIOException.class, () -> spy.renderOptions(sb, 50, options, 2, 5));
+    }
+
     @Test
     public void testRenderWrappedTextMultiLine() {
         // multi line text
@@ -608,7 +715,7 @@ public class HelpFormatterTest {
     }
 
     @Test
-    public void testRenderWrappedTextSingleLine() {
+    public void testRenderWrappedTextSingleLine() throws IOException {
         // single line text
         final int width = 12;
         final int padding = 0;
@@ -618,6 +725,11 @@ public class HelpFormatterTest {
         final StringBuffer sb = new StringBuffer();
         new HelpFormatter().renderWrappedText(sb, width, padding, text);
         assertEquals(expected, sb.toString(), "single line text");
+        // check internal exception handling for coverage
+        final HelpFormatter spy = spy(new HelpFormatter());
+        when(spy.appendWrappedText(sb, width, padding, text)).thenThrow(IOException.class);
+        assertThrows(UncheckedIOException.class, () -> spy.renderWrappedText(sb, width, padding, text));
+
     }
 
     @Test
